@@ -1,5 +1,5 @@
 import click
-from pcbnewTransition import pcbnew, isV6
+from pcbnewTransition import pcbnew
 import csv
 import os
 import sys
@@ -23,10 +23,14 @@ def collectBom(components, lscsFields, ignore):
             continue
         if hasattr(c, "in_bom") and not c.in_bom:
             continue
+        if hasattr(c, "on_board") and not c.on_board:
+            continue
+        if hasattr(c, "dnp") and c.dnp:
+            continue
         orderCode = None
         for fieldName in lscsFields:
             orderCode = getField(c, fieldName)
-            if orderCode is not None:
+            if orderCode is not None and orderCode.strip() != "":
                 break
         cType = (
             getField(c, "Value"),
@@ -37,7 +41,7 @@ def collectBom(components, lscsFields, ignore):
     return bom
 
 def bomToCsv(bomData, filename):
-    with open(filename, "w", newline="") as csvfile:
+    with open(filename, "w", newline="", encoding="utf-8") as csvfile:
         writer = csv.writer(csvfile)
         writer.writerow(["Comment", "Designator", "Footprint", "LCSC"])
         for cType, references in bomData.items():
@@ -45,16 +49,15 @@ def bomToCsv(bomData, filename):
             # the BOM into multiple lines. Let's make the chunks by 100 just to
             # be sure.
             CHUNK_SIZE = 100
+            sortedReferences = sorted(references, key=naturalComponentKey)
             for i in range(0, len(references), CHUNK_SIZE):
-                refChunk = references[i:i+CHUNK_SIZE]
+                refChunk = sortedReferences[i:i+CHUNK_SIZE]
                 value, footprint, lcsc = cType
                 writer.writerow([value, ",".join(refChunk), footprint, lcsc])
 
-def noFilter(footprint):
-    return True
-
 def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
-           corrections, correctionpatterns, missingerror, nametemplate, drc):
+           corrections, correctionpatterns, missingerror, nametemplate, drc,
+           autoname):
     """
     Prepare fabrication files for JLCPCB including their assembly service
     """
@@ -71,7 +74,12 @@ def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
     gerberdir = os.path.join(outputdir, "gerber")
     shutil.rmtree(gerberdir, ignore_errors=True)
     gerberImpl(board, gerberdir)
-    archiveName = nametemplate.format("gerbers")
+
+    if autoname:
+        boardName = os.path.basename(board.replace(".kicad_pcb", ""))
+        archiveName = expandNameTemplate(nametemplate, boardName + "-gerbers", loadedBoard)
+    else:
+        archiveName = expandNameTemplate(nametemplate, "gerbers", loadedBoard)
     shutil.make_archive(os.path.join(outputdir, archiveName), "zip", outputdir, "gerber")
 
     if not assembly:
@@ -86,8 +94,12 @@ def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
     ordercodeFields = [x.strip() for x in field.split(",")]
     bom = collectBom(components, ordercodeFields, refsToIgnore)
 
+    bom_refs = set(x for xs in bom.values() for x in xs)
+    bom_components = [c for c in components if getReference(c) in bom_refs]
+
     posData = collectPosData(loadedBoard, correctionFields,
-        bom=components, posFilter=noFilter, correctionFile=correctionpatterns)
+        bom=bom_components, posFilter=noFilter, correctionFile=correctionpatterns,
+        orientationHandling=FootprintOrientationHandling.MirrorBottom)
     boardReferences = set([x[0] for x in posData])
     bom = {key: [v for v in val if v in boardReferences] for key, val in bom.items()}
     bom = {key: val for key, val in bom.items() if len(val) > 0}
@@ -103,5 +115,5 @@ def exportJlcpcb(board, outputdir, assembly, schematic, ignore, field,
     if missingFields and missingerror:
         sys.exit("There are components with missing ordercode, aborting")
 
-    posDataToFile(posData, os.path.join(outputdir, nametemplate.format("pos") + ".csv"))
-    bomToCsv(bom, os.path.join(outputdir, nametemplate.format("bom") + ".csv"))
+    posDataToFile(posData, os.path.join(outputdir, expandNameTemplate(nametemplate, "pos", loadedBoard) + ".csv"))
+    bomToCsv(bom, os.path.join(outputdir, expandNameTemplate(nametemplate, "bom", loadedBoard) + ".csv"))
